@@ -6,10 +6,10 @@
  * Status:        Experimental.
  * Author:        Dag Brattli <dagb@cs.uit.no>
  * Created at:    Sun Oct  4 20:33:05 1998
- * Modified at:   Sun Sep 26 22:16:25 1999
+ * Modified at:   Wed Jan 19 11:03:32 2000
  * Modified by:   Dag Brattli <dagb@cs.uit.no>
  * 
- *     Copyright (c) 1998-1999 Dag Brattli, All Rights Reserved.
+ *     Copyright (c) 1998-2000 Dag Brattli, All Rights Reserved.
  *      
  *     This program is free software; you can redistribute it and/or 
  *     modify it under the terms of the GNU General Public License as 
@@ -29,20 +29,29 @@
 
 #include <net/if_arp.h>
 #include <net/if_packet.h>
+#include <net/if.h>
 #include <netinet/if_ether.h>
 
 #include <netpacket/packet.h>
 
-#include <linux/irda.h>
+#include <stdint.h>
+#include <irda.h>
 
 #include "irdadump.h"
 
 int config_print_diff = 0;
 int config_dump_frame = 0;
 int config_print_irlmp = 1;
+int config_print_lost_frames = 0;
+
+int self_nr = 0;
+int self_ns = 0;
+int peer_nr = 0;
+int peer_ns = 0;
 
 int verbose = 0;
 int snaplen = 32;
+int ifindex = 0;
 
 GNetBuf *buf;
 struct sockaddr_ll from;
@@ -314,7 +323,59 @@ inline void parse_i_frame(guint8 caddr, guint8 cmd, guint8 pf, int type,
 	
 	/* Check if we should print IrLMP information */
 	if (config_print_irlmp)
-		parse_irlmp(buf, str, type);
+		parse_irlmp(buf, str, type, cmd);
+
+
+	if (config_print_lost_frames) {
+		if (type) {
+			self_nr = nr;
+			if (self_nr != ((peer_ns + 1) % 8)) {
+				g_string_sprintfa(str, 
+						  "** %d frame(s) lost ** ",
+						  (peer_ns + 1 > self_nr) ? 
+						  peer_ns + 1 - self_nr : 
+						  peer_ns + 9 - self_nr);
+			}
+			if (ns != ((self_ns + 1) % 8)) {
+				g_string_append(str, "** retransmit ** ");
+			}
+			self_ns = ns;		
+		} else {
+			peer_nr = nr;
+			if (peer_nr != ((self_ns + 1) % 8)) {
+				g_string_sprintfa(str, 
+						  "** %d frame(s) lost ** ",
+						  (self_ns + 1 > peer_nr) ? 
+						  self_ns + 1 - peer_nr : 
+						  self_ns + 9 - peer_nr);
+			}
+			if (ns != ((peer_ns + 1) % 8)) {
+				g_string_append(str, "** retransmit **");
+			}
+			peer_ns = ns;
+		}
+	}
+}
+
+/*
+ * Function parse_ui_frame (buf, len, u_char)
+ *
+ *    Information frames
+ *
+ */
+inline void parse_ui_frame(guint8 caddr, guint8 cmd, guint8 pf, int type, 
+			   GNetBuf *buf, GString *str)
+{
+	/* Remove IrLAP header */
+	g_netbuf_pull(buf, 2);
+	
+	g_string_sprintfa(str, "ui:%s %s ca=%02x pf=%d  ", 
+			  cmd ? "cmd" : "rsp", 
+			  type ? ">" : "<", caddr, pf);
+	
+	/* Check if we should print IrLMP information */
+	if (config_print_irlmp)
+		parse_ui_irlmp(buf, str, type);
 }
 
 /*
@@ -355,6 +416,28 @@ inline void parse_rr_frame(guint8 caddr, guint8 cmd, guint8 pf, int type,
 	g_string_sprintfa(str, "rr:%s %s ca=%02x pf=%d nr=%d ", 
 			  cmd ? "cmd" : "rsp", type ? ">" : "<", caddr, pf, 
 			  nr);
+
+	if (config_print_lost_frames) {
+		if (type) {
+			self_nr = nr;
+			if (self_nr != ((peer_ns + 1) % 8)) {
+				g_string_sprintfa(str, 
+						  "** %d frame(s) lost ** ",
+						  (peer_ns + 1 > self_nr) ? 
+						  peer_ns + 1 - self_nr : 
+						  peer_ns + 9 - self_nr);
+			}
+		} else {
+			peer_nr = nr;
+			if (peer_nr != ((self_ns + 1) % 8)) {
+				g_string_sprintfa(str, 
+						  "** %d frame(s) lost ** ",
+						  (self_ns + 1 > peer_nr) ? 
+						  self_ns + 1 - peer_nr : 
+						  self_ns + 9 - peer_nr);
+			}
+		}
+	}
 }
 
 /*
@@ -636,7 +719,7 @@ inline void parse_irda_frame(int type, GNetBuf *buf, GString *str)
 		parse_frmr_frame(caddr, cmd, pf, type, buf, str);
 		break;
 	case UI_FRAME:
-		/* parse_ui_frame( self, skb, &info); */
+		parse_ui_frame(caddr, cmd, pf, type, buf, str);
 		break;
 	default:
 	        g_string_sprintfa(str, "Unknown frame %02x received!\n", 
@@ -651,8 +734,10 @@ inline void parse_irda_frame(int type, GNetBuf *buf, GString *str)
  *    
  *
  */
-int irdadump_init(void)
+int irdadump_init(char *ifdev)
 {
+	struct ifreq ifr;
+
 	memset(&last_ias, 0, sizeof(struct ias_query));
 	memset(&conn, 0, sizeof(struct lsap_state));
 	
@@ -671,6 +756,15 @@ int irdadump_init(void)
 		return fd;
         }
 
+	if (ifdev) {
+		strncpy(ifr.ifr_name, ifdev, IFNAMSIZ);
+		if (ioctl(fd, SIOCGIFINDEX, &ifr) < 0) {
+			perror("ioctl");
+			exit(-1);
+		}
+		ifindex = ifr.ifr_ifindex;
+	}
+
 	buf = g_netbuf_new(MAX_FRAME_SIZE);
 
 	return fd;
@@ -684,9 +778,7 @@ int irdadump_init(void)
  */
 int irdadump_loop(GString *str) 
 {
-	int i,maxlen;		
 	int len;
-	char c;
 
 	fromlen = sizeof(struct sockaddr_ll);
 	
@@ -703,13 +795,24 @@ int irdadump_loop(GString *str)
 	if (from.sll_protocol != ntohs(ETH_P_IRDA))
 		return -1;
 
+	/* Filter away frames from other IrDA interfaces */
+	if (ifindex && (from.sll_ifindex != ifindex))
+		return -1;
+
+	/* Filter away empty frames (forced speed change) */
+	if (len == 0)
+		return -1;
+
 	/* Data should be fine now */
 	g_netbuf_put(buf, len);
-			
-	/* Get current time */
-	gettimeofday(curr_time, (struct timezone*) 0);
+
+	/* Get time from packet */
+	if (ioctl(fd, SIOCGSTAMP, curr_time) < 0) {
+		perror("ioctl");
+		exit(-1);
+	}
  	print_time(curr_time, str);
-	
+
 	if (config_print_diff) {
 		print_diff_time(curr_time, prev_time, str);
 		
@@ -723,6 +826,9 @@ int irdadump_loop(GString *str)
         g_string_sprintfa(str, "(%d) ", len);
 
 	if (config_dump_frame) {
+		int i, maxlen;
+		char c;
+
 		g_string_append(str, "\n\t");
 		maxlen = (len < snaplen) ? len : snaplen;
 		
